;; Display the value of a state variable.
;; The argument n specifies the amount of indentation.
(define (display-variable-value state-var assignment-table n if-string)
  (let* ((lhs (make-ast-instance <sal-name-expr> state-var
																 :decl state-var))
				 (value (cond 
								 ((eq-hash-table/get assignment-table state-var) => cdr)
								 (else #f)))
				 (assignments (make-queue))
				 (rname #f))
    (when value
			;; A variable may be a complex data-structure, the 
			;; function sal-value->assignments will break it in
			;; small assignments.
					(sal-value->assignments-core value #f lhs assignments))
;		
; (instance-of? (sal-binary-application/arg1 (car (queue->list assignments))) <sal-record-selection>)
; (sal-selection/target (sal-binary-application/arg1 (car (queue->list assignments))))
    (for-each (lambda (assignment)
								(indent n)
;								(breakpoint "value" (rname assignment) #t)
								(if (instance-of? (sal-binary-application/arg1 assignment) <sal-record-selection>)
										(begin
											(unless
											 (eq? rname (sal-selection/target (sal-binary-application/arg1 assignment)))
											 (set! rname (sal-selection/target (sal-binary-application/arg1 assignment)))
											 (display "INTERFACE ") (sal-value/pp rname)(print ": " if-string) (indent n))
											(indent 4)
											(sal-value/pp
											 (make-sal-equality
												(sal-selection/idx (sal-binary-application/arg1 assignment))
												(sal-binary-application/arg2 assignment)))
											(print ";"))
										(if rname
												(begin
													(set! rname #f)
													(print "END INTERFACE")
													(sal-value/pp assignment))
												(begin
													(sal-value/pp assignment)
													(print ";")))))
							(queue->list assignments))
		(when rname (indent n) (print "END INTERFACE"))))

;; Display the values of the state variables.
;; Only the state variables satisfying the predicate pred? are 
;; displayed. 
;; The argument n specifies the amount of indentation.
(define (display-variable-values state-vars assignment-table n pred? if-string)
  (for-each (lambda (state-var)
							(when (pred? state-var)
								(display-variable-value state-var assignment-table n if-string)))
						state-vars))

(define (display-input-variable-values state-vars assignment-table n)
  (display-variable-values state-vars assignment-table n 
													 (lambda (var) (and (instance-of? var <sal-input-state-var-decl>)
																							(not (instance-of? var <sal-choice-input-state-var-decl>)))) "READER"))

(define (display-output-variable-values state-vars assignment-table n)
  (display-variable-values state-vars assignment-table n 
													 (lambda (var) (instance-of? var <sal-output-state-var-decl>)) "SENDER"))

(define (display-local-variable-values state-vars assignment-table n)
  (display-variable-values state-vars assignment-table n 
													 (lambda (var) (instance-of? var <sal-local-state-var-decl>)) "LOCAL"))

;; Display an execution step.
;; The argument idx is the index of the step.
;; The argument n specifies the amount of indentation.
(define (display-step idx state-vars assignment-table n)
  (indent n) (print "STEP " idx)
  (let ((n (+ n 4))) ;; indent the body of the step
    (indent n) (print "/* Input Variables */")
    (indent n) (print "INPUT")
    (display-input-variable-values state-vars assignment-table (+ n 4))
    (indent n) (print "ENDINPUT")
    (print "")
    
    (indent n) (print "/* Output Variables */")
    (indent n) (print "OUTPUT")
    (display-output-variable-values state-vars assignment-table (+ n 4))
    (indent n) (print "ENDOUTPUT"))

    (indent n) (print "/* State Variables */")
    (indent n) (print "STATE")
    (display-local-variable-values state-vars assignment-table (+ n 4))
    (indent n) (print "ENDSTATE")
    (print "")
    
  (indent n) (print "End Step " idx))

;; (define (sal-path/itr-pp path)
(define (test-path/pp path n)
  ;; Assuming it is not a cyclic path
  (let* ((flat-module (slot-value path :flat-module))
				 (state-vars (slot-value flat-module :state-vars))
				 (step-list (slot-value path :step-info-list)))
    (print "TESTCASE " n)
    (let loop ((step-list step-list)
							 (i 1))
      (unless (null? step-list)
							(let* ((step (car step-list))
										 (assignment-table (slot-value step :assignment-table)))
								(display-step i state-vars assignment-table 4))
							(print "")
							(loop (cdr step-list)
										(+ i 1))))
    (print "ENDTESTCASE " n)))

